/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.dev.jas.as.impl;

import com.google.gwt.dev.js.ast.JsContext;
import com.google.gwt.dev.js.ast.JsExprStmt;
import com.google.gwt.dev.js.ast.JsExpression;
import com.google.gwt.dev.js.ast.JsFunction;
import com.google.gwt.dev.js.ast.JsModVisitor;
import com.google.gwt.dev.js.ast.JsName;
import com.google.gwt.dev.js.ast.JsNode;
import com.google.gwt.dev.js.ast.JsProgram;
import com.google.gwt.dev.js.ast.JsStatement;
import com.google.gwt.dev.js.ast.JsVars;

import java.util.Stack;

/**
 * Changes block-level declarations of the form "function foo() {...}"
 * into "var foo = function() {...}".
 *
 * <p>
 * This is necessary to make the program compatible with ActionScript 2.0.
 */
public final class JsNamedFuncRemover {

  private static class Visitor extends JsModVisitor {

    @Override
    public void endVisit(JsExprStmt node, JsContext ctx) {
      JsExpression expr = node.getExpression();
      if (!(expr instanceof JsFunction))
        return;
      JsFunction func = (JsFunction)expr;
      JsName name = func.getName();
      if (name == null)
        return;
      func.setName(null);
      JsVars.JsVar var = new JsVars.JsVar(node.getSourceInfo(), name);
      var.setInitExpr(func);
      JsVars vars = new JsVars(node.getSourceInfo());
      vars.add(var);
      ctx.replaceMe(vars);
    }
  }

  public static void exec(JsProgram program) {
    new Visitor().accept(program);
  }

  private JsNamedFuncRemover() {
  }
}
