/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.dev.jas.as.impl;

import com.google.gwt.dev.js.ast.JsContext;
import com.google.gwt.dev.js.ast.JsExpression;
import com.google.gwt.dev.js.ast.JsModVisitor;
import com.google.gwt.dev.js.ast.JsObjectLiteral;
import com.google.gwt.dev.js.ast.JsProgram;
import com.google.gwt.dev.js.ast.JsRegExp;

/**
 * Eliminates uses of regular expression literals, replacing them with an
 * empty object. We have no equivalent for them in ActionScript 2.0.
 *
 * <p>
 * This is necessary to make the program compatible with ActionScript 2.0.
 */
public final class JsRegExpRemover {

  private static class Visitor extends JsModVisitor {

    @Override
    public void endVisit(JsRegExp regexp, JsContext ctx) {
      ctx.replaceMe(new JsObjectLiteral(regexp.getSourceInfo()));
    }
  }

  public static void exec(JsProgram program) {
    new Visitor().accept(program);
  }

  private JsRegExpRemover() {
  }
}
