/*
 * Copyright 2011 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.jas.linker;

import com.google.gwt.core.ext.Linker;
import com.google.gwt.core.ext.linker.Artifact;
import com.google.gwt.core.ext.linker.Transferable;

import java.util.Set;

/**
 * Records Flash Import statements.
 */
@Transferable
public class FlashImportsArtifact extends
    Artifact<FlashImportsArtifact> {

  private Set<String> imports;

  public Set<String> getImports() {
    return imports;
  }

  public FlashImportsArtifact(Class<? extends Linker> linker, Set<String>
       imports) {
    super(linker);
    this.imports = imports;
  }

  @Override
  public int hashCode() {
    return getName().hashCode();
  }

  private String getName() {
    return getComparableArtifactType().getSimpleName();
  }

  @Override
  protected int compareToComparableArtifact(FlashImportsArtifact o) {
    return getName().compareTo(o.getName());
  }

  @Override
  protected Class<FlashImportsArtifact> getComparableArtifactType() {
    return FlashImportsArtifact.class;
  }
}
